@@ -12,10 +12,7 @@ This file only deals with non-GUI configuration features
 sip API incompatibility issue in spyder's non-gui modules)
 """
 
-from __future__ import print_function
-
 import codecs
-import getpass
 import locale
 import os
 import os.path as osp
@@ -68,6 +65,11 @@ def running_in_ci():
     return bool(os.environ.get('CI'))
 
 
+def running_in_ci_with_conda():
+    """Return True if currently running under CI with conda packages."""
+    return running_in_ci() and bool(os.environ.get('USE_CONDA'))
+
+
 def is_stable_version(version):
     """
     Return true if version is stable, i.e. with letters in the final component.
@@ -398,9 +400,10 @@ def get_available_translations():
     # is added, to ensure LANGUAGE_CODES is updated.
     for lang in langs:
         if lang not in LANGUAGE_CODES:
-            error = ('Update LANGUAGE_CODES (inside config/base.py) if a new '
-                     'translation has been added to Spyder')
-            print(error)  # spyder: test-skip
+            if DEV:
+                error = ('Update LANGUAGE_CODES (inside config/base.py) if a '
+                         'new translation has been added to Spyder')
+                print(error)  # spyder: test-skip
             return ['en']
     return langs
 
