@@ -15,6 +15,8 @@ from unittest.mock import Mock
 # Third party imports
 import pytest
 from qtpy.QtGui import QFont
+from qtpy.QtCore import QMimeData, QUrl
+from qtpy.QtWidgets import QApplication
 
 # Local imports
 from spyder.config.manager import CONF
@@ -212,3 +214,30 @@ def search_codeeditor(completions_codeeditor, qtbot_module, request):
     request.addfinalizer(teardown)
 
     return code_editor, find_replace
+
+
+@pytest.fixture
+def codeeditor(qtbot):
+    widget = CodeEditor(None)
+    widget.setup_editor(linenumbers=True,
+                        markers=True,
+                        tab_mode=False,
+                        font=QFont("Courier New", 10),
+                        show_blanks=True, color_scheme='spyder/dark',
+                        scroll_past_end=True)
+    widget.setup_editor(language='Python')
+    widget.resize(640, 480)
+    widget.show()
+    yield widget
+    widget.close()
+
+
+@pytest.fixture
+def copy_files_clipboard(create_folders_files):
+    """Fixture to copy files/folders into the clipboard"""
+    file_paths = create_folders_files[0]
+    file_content = QMimeData()
+    file_content.setUrls([QUrl.fromLocalFile(fname) for fname in file_paths])
+    cb = QApplication.clipboard()
+    cb.setMimeData(file_content, mode=cb.Clipboard)
+    return file_paths
