@@ -125,8 +125,7 @@ class Help(SpyderDockablePlugin):
         shortcuts = self.get_plugin(Plugins.Shortcuts)
 
         # See: spyder-ide/spyder#6992
-        self._show_intro_message = lambda: self.show_intro_message()
-        shortcuts.sig_shortcuts_updated.connect(self._show_intro_message)
+        shortcuts.sig_shortcuts_updated.connect(self.show_intro_message)
 
         if self.is_plugin_available(Plugins.MainMenu):
             self._setup_menus()
@@ -173,7 +172,7 @@ class Help(SpyderDockablePlugin):
     @on_plugin_teardown(plugin=Plugins.Shortcuts)
     def on_shortcuts_teardown(self):
         shortcuts = self.get_plugin(Plugins.Shortcuts)
-        shortcuts.sig_shortcuts_updated.disconnect(self._show_intro_message)
+        shortcuts.sig_shortcuts_updated.disconnect(self.show_intro_message)
 
     @on_plugin_teardown(plugin=Plugins.MainMenu)
     def on_main_menu_teardown(self):
@@ -203,6 +202,12 @@ class Help(SpyderDockablePlugin):
         except SpyderAPIError:
             pass
 
+    def on_mainwindow_visible(self):
+        # Raise plugin the first time Spyder starts
+        if self.get_conf('show_first_time', default=True):
+            self.dockwidget.raise_()
+            self.set_conf('show_first_time', False)
+
     # --- Private API
     # ------------------------------------------------------------------------
     def _setup_menus(self):
@@ -279,7 +284,6 @@ class Help(SpyderDockablePlugin):
 
     def show_intro_message(self):
         """Show the IPython introduction message."""
-        self.switch_to_plugin()
         self.get_widget().show_intro_message()
 
     def show_rich_text(self, text, collapse=False, img_path=''):
