@@ -15,10 +15,13 @@ import codecs
 import glob
 import os
 import os.path as osp
+import psutil
 import shutil
 import sys
 import tempfile
 from textwrap import dedent
+import threading
+import traceback
 from unittest.mock import Mock
 
 # Third party imports
@@ -36,7 +39,8 @@ from qtpy.QtWidgets import QMessageBox, QMainWindow
 import sympy
 
 # Local imports
-from spyder.config.base import get_home_dir, running_in_ci
+from spyder.config.base import (
+    get_home_dir, running_in_ci, running_in_ci_with_conda)
 from spyder.config.gui import get_color_scheme
 from spyder.config.manager import ConfigurationManager
 from spyder.py3compat import PY2, to_text_string
@@ -148,7 +152,7 @@ def ipyconsole(qtbot, request, tmpdir):
     # Use the Tkinter backend if requested
     tk_backend = request.node.get_closest_marker('tk_backend')
     if tk_backend:
-        configuration.set('ipython_console', 'pylab/backend', 8)
+        configuration.set('ipython_console', 'pylab/backend', 3)
 
     # Start a Pylab client if requested
     pylab_client = request.node.get_closest_marker('pylab_client')
@@ -201,11 +205,41 @@ def ipyconsole(qtbot, request, tmpdir):
     # Set exclamation mark to True
     configuration.set('ipython_console', 'pdb_use_exclamation_mark', True)
 
-    # This segfaults on macOS
-    if not sys.platform == "darwin":
+    if os.name == 'nt':
         qtbot.addWidget(window)
-    window.resize(640, 480)
-    window.show()
+
+    with qtbot.waitExposed(window):
+        window.resize(640, 480)
+        window.show()
+
+    # Wait until the window is fully up
+    qtbot.waitUntil(lambda: console.get_current_shellwidget() is not None)
+    shell = console.get_current_shellwidget()
+    try:
+        qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                        timeout=SHELL_TIMEOUT)
+    except Exception:
+        # Print content of shellwidget and close window
+        print(console.get_current_shellwidget(
+            )._control.toPlainText())
+        client = console.get_current_client()
+        if client.info_page != client.blank_page:
+            print('info_page')
+            print(client.info_page)
+        raise
+
+    # Check for thread or open file leaks
+    known_leak = request.node.get_closest_marker('known_leak')
+
+    if os.name != 'nt' and not known_leak:
+        # _DummyThread are created if current_thread() is called from them.
+        # They will always leak (From python doc) so we ignore them.
+        init_threads = [
+            repr(thread) for thread in threading.enumerate()
+            if not isinstance(thread, threading._DummyThread)]
+        proc = psutil.Process()
+        init_files = [repr(f) for f in proc.open_files()]
+        init_subprocesses = [repr(f) for f in proc.children()]
 
     yield console
 
@@ -222,11 +256,67 @@ def ipyconsole(qtbot, request, tmpdir):
 
     # Close
     console.on_close()
-    window.close()
     os.environ.pop('IPYCONSOLE_TESTING')
     os.environ.pop('IPYCONSOLE_TEST_DIR')
     os.environ.pop('IPYCONSOLE_TEST_NO_STDERR')
 
+    if os.name == 'nt' or known_leak:
+        # Do not test for leaks
+        return
+
+    def show_diff(init_list, now_list, name):
+        sys.stderr.write(f"Extra {name} before test:\n")
+        for item in init_list:
+            if item in now_list:
+                now_list.remove(item)
+            else:
+                sys.stderr.write(item + "\n")
+        sys.stderr.write(f"Extra {name} after test:\n")
+        for item in now_list:
+            sys.stderr.write(item + "\n")
+
+    # The test is not allowed to open new files or threads.
+    try:
+        def threads_condition():
+            threads = [
+                thread for thread in threading.enumerate()
+                if not isinstance(thread, threading._DummyThread)]
+            return (len(init_threads) >= len(threads))
+
+        qtbot.waitUntil(threads_condition, timeout=SHELL_TIMEOUT)
+    except Exception:
+        now_threads = [
+            thread for thread in threading.enumerate()
+            if not isinstance(thread, threading._DummyThread)]
+        threads = [repr(t) for t in now_threads]
+        show_diff(init_threads, threads, "thread")
+        sys.stderr.write("Running Threads stacks:\n")
+        now_thread_ids = [t.ident for t in now_threads]
+        for threadId, frame in sys._current_frames().items():
+            if threadId in now_thread_ids:
+                sys.stderr.write("\nThread " + str(threads) + ":\n")
+                traceback.print_stack(frame)
+        raise
+
+    try:
+        # -1 from closed client
+        qtbot.waitUntil(lambda: (
+            len(init_subprocesses) - 1 >= len(proc.children())),
+            timeout=SHELL_TIMEOUT)
+    except Exception:
+        subprocesses = [repr(f) for f in proc.children()]
+        show_diff(init_subprocesses, subprocesses, "processes")
+        raise
+
+    try:
+        qtbot.waitUntil(
+            lambda: (len(init_files) >= len(proc.open_files())),
+            timeout=SHELL_TIMEOUT)
+    except Exception:
+        files = [repr(f) for f in proc.open_files()]
+        show_diff(init_files, files, "files")
+        raise
+
 
 # =============================================================================
 # Tests
@@ -237,8 +327,6 @@ def test_banners(ipyconsole, qtbot):
     """Test that console banners are generated correctly."""
     shell = ipyconsole.get_current_shellwidget()
     control = shell._control
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Long banner
     text = control.toPlainText().splitlines()
@@ -280,8 +368,6 @@ def test_get_calltips(ipyconsole, qtbot, function, signature, documentation):
     """Test that calltips show the documentation."""
     shell = ipyconsole.get_current_shellwidget()
     control = shell._control
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Import numpy
     with qtbot.waitSignal(shell.executed):
@@ -310,12 +396,13 @@ def test_get_calltips(ipyconsole, qtbot, function, signature, documentation):
 
 @flaky(max_runs=3)
 @pytest.mark.auto_backend
+@pytest.mark.skipif(
+    running_in_ci() and not os.name == 'nt',
+    reason="Times out on Linux and macOS")
 def test_auto_backend(ipyconsole, qtbot):
     """Test that the automatic backend was set correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     with qtbot.waitSignal(shell.executed):
         shell.execute("ip = get_ipython(); ip.kernel.eventloop")
@@ -330,6 +417,9 @@ def test_auto_backend(ipyconsole, qtbot):
 
 @flaky(max_runs=3)
 @pytest.mark.tk_backend
+@pytest.mark.skipif(
+    running_in_ci() and not os.name == 'nt',
+    reason="Times out on Linux and macOS")
 def test_tk_backend(ipyconsole, qtbot):
     """Test that the Tkinter backend was set correctly."""
     # Wait until the window is fully up
@@ -351,8 +441,6 @@ def test_pylab_client(ipyconsole, qtbot):
     """Test that the Pylab console is working correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # This is here to generate further errors
     with qtbot.waitSignal(shell.executed):
@@ -383,8 +471,6 @@ def test_sympy_client(ipyconsole, qtbot):
     """Test that the SymPy console is working correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # This is here to generate further errors
     with qtbot.waitSignal(shell.executed):
@@ -417,11 +503,9 @@ def test_cython_client(ipyconsole, qtbot):
     """Test that the Cython console is working correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # This is here to generate further errors
-    with qtbot.waitSignal(shell.executed):
+    with qtbot.waitSignal(shell.executed, timeout=SHELL_TIMEOUT):
         shell.execute("%%cython\n"
                       "cdef int ctest(int x, int y):\n"
                       "    return x + y")
@@ -435,7 +519,7 @@ def test_cython_client(ipyconsole, qtbot):
     qtbot.wait(1000)
 
     # See that cython is still enabled after reset
-    with qtbot.waitSignal(shell.executed):
+    with qtbot.waitSignal(shell.executed, timeout=SHELL_TIMEOUT):
         shell.execute("%%cython\n"
                       "cdef int ctest(int x, int y):\n"
                       "    return x + y")
@@ -448,11 +532,6 @@ def test_cython_client(ipyconsole, qtbot):
 @flaky(max_runs=3)
 def test_tab_rename_for_slaves(ipyconsole, qtbot):
     """Test slave clients are renamed correctly."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
-
     cf = ipyconsole.get_current_client().connection_file
     ipyconsole.get_widget()._create_client_for_kernel(cf, None, None, None)
     qtbot.waitUntil(lambda: len(ipyconsole.get_clients()) == 2)
@@ -468,9 +547,6 @@ def test_tab_rename_for_slaves(ipyconsole, qtbot):
 @flaky(max_runs=3)
 def test_no_repeated_tabs_name(ipyconsole, qtbot):
     """Test that tabs can't have repeated given names."""
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
-
     # Rename first client
     ipyconsole.get_widget().rename_tabs_after_change('foo')
 
@@ -484,11 +560,13 @@ def test_no_repeated_tabs_name(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
+@pytest.mark.skipif(
+    running_in_ci() and sys.platform == 'darwin',
+    reason="Hangs sometimes on macOS")
+@pytest.mark.skipif(os.name == 'nt' and running_in_ci_with_conda(),
+                    reason="It hangs on Windows CI using conda")
 def test_tabs_preserve_name_after_move(ipyconsole, qtbot):
     """Test that tabs preserve their names after they are moved."""
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
-
     # Create a new client
     ipyconsole.create_new_client()
 
@@ -505,8 +583,6 @@ def test_conf_env_vars(ipyconsole, qtbot):
     """Test that kernels have env vars set by our kernel spec."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Get a CONF env var
     with qtbot.waitSignal(shell.executed):
@@ -522,8 +598,6 @@ def test_no_stderr_file(ipyconsole, qtbot):
     """Test that consoles can run without an stderr."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Execute a simple assignment
     with qtbot.waitSignal(shell.executed):
@@ -540,8 +614,6 @@ def test_non_ascii_stderr_file(ipyconsole, qtbot):
     """Test the creation of a console with a stderr file in a non-ascii dir."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Execute a simple assignment
     with qtbot.waitSignal(shell.executed):
@@ -558,8 +630,6 @@ def test_console_import_namespace(ipyconsole, qtbot):
     """Test an import of the form 'from foo import *'."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Import numpy
     with qtbot.waitSignal(shell.executed):
@@ -572,9 +642,6 @@ def test_console_import_namespace(ipyconsole, qtbot):
 @flaky(max_runs=3)
 def test_console_disambiguation(ipyconsole, qtbot):
     """Test the disambiguation of dedicated consoles."""
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
-
     # Create directories and file for TEMP_DIRECTORY/a/b/c.py
     # and TEMP_DIRECTORY/a/d/c.py
     dir_b = osp.join(TEMP_DIRECTORY, 'a', 'b')
@@ -609,9 +676,6 @@ def test_console_disambiguation(ipyconsole, qtbot):
 @flaky(max_runs=3)
 def test_console_coloring(ipyconsole, qtbot):
     """Test that console gets the same coloring present in the Editor."""
-    shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
-
     config_options = ipyconsole.get_widget().config_options()
 
     syntax_style = config_options.JupyterWidget.syntax_style
@@ -639,8 +703,6 @@ def test_set_cwd(ipyconsole, qtbot, tmpdir):
     """Test kernel when changing cwd."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # spyder-ide/spyder#6451.
     savetemp = shell._cwd
@@ -663,8 +725,6 @@ def test_get_cwd(ipyconsole, qtbot, tmpdir):
     """Test current working directory."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # spyder-ide/spyder#6451.
     savetemp = shell._cwd
@@ -695,7 +755,6 @@ def test_get_cwd(ipyconsole, qtbot, tmpdir):
 def test_request_env(ipyconsole, qtbot):
     """Test that getting env vars from the kernel is working as expected."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Add a new entry to os.environ
     with qtbot.waitSignal(shell.executed):
@@ -721,7 +780,6 @@ def test_request_syspath(ipyconsole, qtbot, tmpdir):
     expected.
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Add a new entry to sys.path
     with qtbot.waitSignal(shell.executed):
@@ -744,8 +802,6 @@ def test_request_syspath(ipyconsole, qtbot, tmpdir):
 def test_save_history_dbg(ipyconsole, qtbot):
     """Test that browsing command history is working while debugging."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -823,8 +879,6 @@ def test_save_history_dbg(ipyconsole, qtbot):
 def test_dbg_input(ipyconsole, qtbot):
     """Test that spyder doesn't send pdb commands to unrelated input calls."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -857,7 +911,6 @@ def test_unicode_vars(ipyconsole, qtbot):
     """
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Set value for a Unicode variable
     with qtbot.waitSignal(shell.executed):
@@ -877,9 +930,7 @@ def test_read_stderr(ipyconsole, qtbot):
     """
     Test the read operation of the stderr file of the kernel
     """
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Set contents of the stderr file of the kernel
     content = 'Test text'
@@ -893,14 +944,12 @@ def test_read_stderr(ipyconsole, qtbot):
 @pytest.mark.no_xvfb
 @pytest.mark.skipif(running_in_ci() and os.name == 'nt',
                     reason="Times out on Windows")
-@pytest.mark.skipif(PY2, reason="It times out in Python 2.")
 def test_values_dbg(ipyconsole, qtbot):
     """
     Test that getting, setting, copying and removing values is working while
     debugging.
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -950,7 +999,6 @@ def test_execute_events_dbg(ipyconsole, qtbot):
     """Test execute events while debugging"""
 
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1003,7 +1051,6 @@ def test_run_doctest(ipyconsole, qtbot):
     Test that doctests can be run without problems
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     code = dedent('''
     def add(x, y):
@@ -1041,7 +1088,6 @@ def test_mpl_backend_change(ipyconsole, qtbot):
     using the %matplotlib magic
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Import Matplotlib
     with qtbot.waitSignal(shell.executed):
@@ -1065,12 +1111,15 @@ def test_mpl_backend_change(ipyconsole, qtbot):
 
 @flaky(max_runs=10)
 @pytest.mark.skipif(running_in_ci(), reason="Fails frequently in CI")
+@pytest.mark.skipif(os.name == 'nt',
+                    reason="Failing on Windows. In fact on Windows you can "
+                           "copy with Ctrl + C i.e a KeyboardInterrupt isn't "
+                           "triggered when doing Ctrl + C while debugging")
 def test_ctrl_c_dbg(ipyconsole, qtbot):
     """
     Test that Ctrl+C works while debugging
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1081,6 +1130,8 @@ def test_ctrl_c_dbg(ipyconsole, qtbot):
         shell.execute('%debug print()')
 
     # Test Ctrl+C
+    qtbot.waitUntil(lambda: 'IPdb [1]: ' in control.toPlainText())
+    control.setFocus()
     qtbot.keyClick(control, Qt.Key_C, modifier=Qt.ControlModifier)
     qtbot.waitUntil(
         lambda: 'For copying text while debugging, use Ctrl+Shift+C' in
@@ -1096,7 +1147,6 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     Test that clear and reset magics are working while debugging
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1134,19 +1184,31 @@ def test_restart_kernel(ipyconsole, mocker, qtbot):
     # Mock method we want to check
     mocker.patch.object(ClientWidget, "_show_mpl_backend_errors")
 
+    ipyconsole.create_new_client()
+
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
 
     # Do an assignment to verify that it's not there after restarting
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 10')
 
+    # Write something to stderr to verify that it's not there after restarting
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('import sys; sys.__stderr__.write("HEL"+"LO")')
+
+    qtbot.waitUntil(
+        lambda: 'HELLO' in shell._control.toPlainText(), timeout=SHELL_TIMEOUT)
+
     # Restart kernel and wait until it's up again
     shell._prompt_html = None
     ipyconsole.restart_kernel()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+    qtbot.waitUntil(
+        lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     assert 'Restarting kernel...' in shell._control.toPlainText()
+    assert 'HELLO' not in shell._control.toPlainText()
     assert not shell.is_defined('a')
 
     # Check that we try to show Matplotlib backend errors at the beginning and
@@ -1159,9 +1221,7 @@ def test_load_kernel_file_from_id(ipyconsole, qtbot):
     """
     Test that a new client is created using its id
     """
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     connection_file = osp.basename(client.connection_file)
     id_ = connection_file.split('kernel-')[-1].split('.json')[0]
@@ -1179,9 +1239,7 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot, tmpdir):
     Test that a new client is created using a connection file
     placed in a different location from jupyter_runtime_dir
     """
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     fname = osp.basename(client.connection_file)
     connection_file = to_text_string(tmpdir.join(fname))
@@ -1201,8 +1259,6 @@ def test_load_kernel_file(ipyconsole, qtbot, tmpdir):
     """
     shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     ipyconsole.get_widget()._create_client_for_kernel(
         client.connection_file, None, None, None)
@@ -1224,7 +1280,6 @@ def test_load_kernel_file(ipyconsole, qtbot, tmpdir):
 def test_sys_argv_clear(ipyconsole, qtbot):
     """Test that sys.argv is cleared up correctly"""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     with qtbot.waitSignal(shell.executed):
         shell.execute('import sys; A = sys.argv')
@@ -1234,12 +1289,10 @@ def test_sys_argv_clear(ipyconsole, qtbot):
 
 
 @flaky(max_runs=5)
+@pytest.mark.skipif(os.name == 'nt', reason="Fails sometimes on Windows")
 def test_set_elapsed_time(ipyconsole, qtbot):
     """Test that the IPython console elapsed timer is set correctly."""
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Show time label.
     ipyconsole.get_widget().set_show_elapsed_time_current_client(True)
@@ -1270,11 +1323,7 @@ def test_set_elapsed_time(ipyconsole, qtbot):
 @pytest.mark.skipif(os.name == 'nt', reason="Doesn't work on Windows")
 def test_stderr_file_is_removed_one_kernel(ipyconsole, qtbot, monkeypatch):
     """Test that consoles removes stderr when client is closed."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # In a normal situation file should exist
     monkeypatch.setattr(QMessageBox, 'question',
@@ -1285,15 +1334,13 @@ def test_stderr_file_is_removed_one_kernel(ipyconsole, qtbot, monkeypatch):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="Doesn't work on Windows")
+@pytest.mark.skipif(
+    not sys.platform.startswith('linux'),
+    reason="Doesn't work on Windows and hangs sometimes on Mac")
 def test_stderr_file_is_removed_two_kernels(ipyconsole, qtbot, monkeypatch):
     """Test that console removes stderr when client and related clients
     are closed."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # New client with the same kernel
     ipyconsole.get_widget()._create_client_for_kernel(
@@ -1315,11 +1362,7 @@ def test_stderr_file_is_removed_two_kernels(ipyconsole, qtbot, monkeypatch):
 def test_stderr_file_remains_two_kernels(ipyconsole, qtbot, monkeypatch):
     """Test that console doesn't remove stderr when a related client is not
     closed."""
-    # Wait until the window is fully up
-    shell = ipyconsole.get_current_shellwidget()
     client = ipyconsole.get_current_client()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # New client with the same kernel
     ipyconsole.get_widget()._create_client_for_kernel(
@@ -1345,30 +1388,33 @@ def test_kernel_crash(ipyconsole, qtbot):
     # Create an IPython kernel config file with a bad config
     ipy_kernel_cfg = osp.join(get_ipython_dir(), 'profile_default',
                               'ipython_kernel_config.py')
-    with open(ipy_kernel_cfg, 'w') as f:
-        # This option must be a string, not an int
-        f.write("c.InteractiveShellApp.extra_extension = 1")
+    try:
+        with open(ipy_kernel_cfg, 'w') as f:
+            # This option must be a string, not an int
+            f.write("c.InteractiveShellApp.extra_extension = 1")
 
-    ipyconsole.create_new_client()
+        ipyconsole.get_widget().close_cached_kernel()
+        ipyconsole.create_new_client()
 
-    # Assert that the console is showing an error
-    qtbot.waitUntil(lambda: ipyconsole.get_clients()[-1].is_error_shown,
-                    timeout=6000)
-    error_client = ipyconsole.get_clients()[-1]
-    assert error_client.is_error_shown
+        # Assert that the console is showing an error
+        qtbot.waitUntil(lambda: ipyconsole.get_clients()[-1].is_error_shown,
+                        timeout=6000)
+        error_client = ipyconsole.get_clients()[-1]
+        assert error_client.is_error_shown
 
-    # Assert the error contains the text we expect
-    webview = error_client.infowidget
-    if WEBENGINE:
-        webpage = webview.page()
-    else:
-        webpage = webview.page().mainFrame()
-    qtbot.waitUntil(
-        lambda: check_text(webpage, "Bad config encountered"),
-        timeout=6000)
+        # Assert the error contains the text we expect
+        webview = error_client.infowidget
+        if WEBENGINE:
+            webpage = webview.page()
+        else:
+            webpage = webview.page().mainFrame()
 
-    # Remove bad kernel config file
-    os.remove(ipy_kernel_cfg)
+        qtbot.waitUntil(
+            lambda: check_text(webpage, "Bad config encountered"),
+            timeout=6000)
+    finally:
+        # Remove bad kernel config file
+        os.remove(ipy_kernel_cfg)
 
 
 @flaky(max_runs=3)
@@ -1392,21 +1438,21 @@ def test_remove_old_std_files(ipyconsole, qtbot):
 
     # The current kernel std files should be present
     for fname in glob.glob(osp.join(tmpdir, '*')):
-        assert osp.basename(fname).startswith('kernel')
-        assert any(
-            [osp.basename(fname).endswith(ext)
-             for ext in ('.stderr', '.stdout', '.fault')]
-        )
+        if osp.basename(fname) != 'test':
+            assert osp.basename(fname).startswith('kernel')
+            assert any(
+                [osp.basename(fname).endswith(ext)
+                 for ext in ('.stderr', '.stdout', '.fault')]
+            )
 
 
 @flaky(max_runs=10)
 @pytest.mark.use_startup_wdir
-@pytest.mark.skipif(os.name == 'nt', reason="Too flaky on Windows")
+@pytest.mark.skipif(sys.platform.startswith('linux'),
+                    reason="Too flaky on Linux")
 def test_console_working_directory(ipyconsole, qtbot):
     """Test for checking the working directory."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
     with qtbot.waitSignal(shell.executed):
         shell.execute('import os; cwd = os.getcwd()')
 
@@ -1421,8 +1467,6 @@ def test_console_working_directory(ipyconsole, qtbot):
 def test_console_complete(ipyconsole, qtbot, tmpdir):
     """Test code completions in the console."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1553,8 +1597,6 @@ def test_console_complete(ipyconsole, qtbot, tmpdir):
 def test_pdb_multiline(ipyconsole, qtbot):
     """Test entering a multiline statment into pdb"""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1585,8 +1627,6 @@ def test_pdb_multiline(ipyconsole, qtbot):
 def test_pdb_ignore_lib(ipyconsole, qtbot, show_lib):
     """Test that pdb can avoid closed files."""
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1622,8 +1662,6 @@ def test_calltip(ipyconsole, qtbot):
     See spyder-ide/spyder#10842
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Give focus to the widget that's going to receive clicks
     control = ipyconsole.get_widget().get_focus_widget()
@@ -1645,8 +1683,6 @@ def test_conda_env_activation(ipyconsole, qtbot):
     """
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
 
     # Get conda activation environment variable
     with qtbot.waitSignal(shell.executed):
@@ -1666,8 +1702,6 @@ def test_kernel_kill(ipyconsole, qtbot):
     Test that the kernel correctly restarts after a kill.
     """
     shell = ipyconsole.get_current_shellwidget()
-    qtbot.waitUntil(lambda: shell._prompt_html is not None,
-                    timeout=SHELL_TIMEOUT)
     # Wait for the restarter to start
     qtbot.wait(3000)
     crash_string = 'import os, signal; os.kill(os.getpid(), signal.SIGTERM)'
@@ -1734,6 +1768,7 @@ def test_wrong_std_module(ipyconsole, qtbot, tmpdir, spyder_pythonpath):
 
 
 @flaky(max_runs=3)
+@pytest.mark.known_leak
 @pytest.mark.skipif(os.name == 'nt', reason="no SIGTERM on Windows")
 def test_kernel_restart_after_manual_restart_and_crash(ipyconsole, qtbot):
     """
@@ -1847,14 +1882,16 @@ def test_startup_code_pdb(ipyconsole, qtbot):
 @flaky(max_runs=3)
 @pytest.mark.parametrize(
     "backend",
-    ['inline', 'qt5', 'tk', 'osx', ]
+    ['inline', 'qt5', 'tk', 'osx']
 )
 def test_pdb_eventloop(ipyconsole, qtbot, backend):
-    """Check if pdb works with every backend. (only testing 3)."""
+    """Check if setting an event loop while debugging works."""
     # Skip failing tests
-    if backend == 'tk' and (os.name == 'nt' or PY2):
+    if backend == 'tk' and os.name == 'nt':
         return
-    if backend == 'osx' and (sys.platform != "darwin" or PY2):
+    if backend == 'osx' and sys.platform != "darwin":
+        return
+    if backend == 'qt5' and not os.name == "nt" and running_in_ci():
         return
 
     shell = ipyconsole.get_current_shellwidget()
@@ -1949,11 +1986,10 @@ def test_stop_pdb(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(sys.platform == 'nt', reason="Times out on Windows")
 def test_code_cache(ipyconsole, qtbot):
     """
     Test that code sent to execute is properly cached
-    and that the cache is empited on interrupt.
+    and that the cache is emptied on interrupt.
     """
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
@@ -1971,7 +2007,8 @@ def test_code_cache(ipyconsole, qtbot):
 
     # Send two execute requests and make sure the second one is executed
     shell.execute('import time; time.sleep(.5)')
-    shell.execute('var = 142')
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('var = 142')
     qtbot.wait(500)
     qtbot.waitUntil(lambda: check_value('var', 142))
     assert shell.get_value('var') == 142
@@ -2063,9 +2100,51 @@ def test_breakpoint_builtin(ipyconsole, qtbot, tmpdir):
     assert 'IPdb [1]:' in control.toPlainText()
 
 
+def test_pdb_out(ipyconsole, qtbot):
+    """Test that browsing command history is working while debugging."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Give focus to the widget that's going to receive clicks
+    control = ipyconsole.get_widget().get_focus_widget()
+    control.setFocus()
+
+    # Enter debugging mode
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('%debug print()')
+
+    # Generate some output
+    with qtbot.waitSignal(shell.executed):
+        shell.pdb_execute('a = 12 + 1; a')
+
+    assert "[1]: 13" in control.toPlainText()
+
+    # Generate hide output
+    with qtbot.waitSignal(shell.executed):
+        shell.pdb_execute('a = 14 + 1; a;')
+
+    assert "[2]: 15" not in control.toPlainText()
+
+    # Multiline
+    with qtbot.waitSignal(shell.executed):
+        shell.pdb_execute('a = 16 + 1\na')
+
+    assert "[3]: 17" in control.toPlainText()
+
+    with qtbot.waitSignal(shell.executed):
+        shell.pdb_execute('a = 18 + 1\na;')
+
+    assert "[4]: 19" not in control.toPlainText()
+    assert "IPdb [4]:" in control.toPlainText()
+
+
 @flaky(max_runs=3)
 @pytest.mark.auto_backend
-def test_shutdown_kernel(ipyconsole, qtbot, tmpdir):
+@pytest.mark.skipif(
+    running_in_ci() and not os.name == 'nt',
+    reason="Times out on Linux and macOS")
+def test_shutdown_kernel(ipyconsole, qtbot):
     """
     Check that the kernel is shutdown after creating plots with the
     automatic backend.
@@ -2103,5 +2182,70 @@ def test_shutdown_kernel(ipyconsole, qtbot, tmpdir):
     assert not shell.get_value('kernel_exists')
 
 
+def test_pdb_comprehension_namespace(ipyconsole, qtbot, tmpdir):
+    """Check that the debugger handles the namespace of a comprehension."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+    control = ipyconsole.get_widget().get_focus_widget()
+
+    # Code to run
+    code = "locals = 1\nx = [locals + i for i in range(2)]"
+
+    # Write code to file on disk
+    file = tmpdir.join('test_breakpoint.py')
+    file.write(code)
+
+    # Run file
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(f"debugfile(filename=r'{str(file)}')")
+
+    # steps 4 times
+    for i in range(4):
+        with qtbot.waitSignal(shell.executed):
+            shell.pdb_execute("s")
+    assert "Error" not in control.toPlainText()
+
+    with qtbot.waitSignal(shell.executed):
+        shell.pdb_execute("print('test', locals + i + 10)")
+
+    assert "Error" not in control.toPlainText()
+    assert "test 11" in control.toPlainText()
+
+    settings = {
+     'check_all': False,
+     'exclude_callables_and_modules': True,
+     'exclude_capitalized': False,
+     'exclude_private': True,
+     'exclude_unsupported': False,
+     'exclude_uppercase': True,
+     'excluded_names': [],
+     'minmax': False,
+     'show_callable_attributes': True,
+     'show_special_attributes': False}
+
+    shell.call_kernel(
+            interrupt=True
+        ).set_namespace_view_settings(settings)
+    namespace = shell.call_kernel(blocking=True).get_namespace_view()
+    for key in namespace:
+        assert "_spyderpdb" not in key
+
+
+@flaky(max_runs=3)
+@pytest.mark.auto_backend
+@pytest.mark.skipif(
+    running_in_ci() and not os.name == 'nt',
+    reason="Times out on Linux and macOS")
+def test_restart_intertactive_backend(ipyconsole):
+    """
+    Test that we ask for a restart after switching to a different interactive
+    backend in preferences.
+    """
+    main_widget = ipyconsole.get_widget()
+    main_widget.change_possible_restart_conf('pylab/backend', 3)
+    assert bool(os.environ.get('BACKEND_REQUIRE_RESTART'))
+
+
 if __name__ == "__main__":
     pytest.main()
