@@ -140,7 +140,7 @@ class Projects(SpyderDockablePlugin):
     # ------------------------------------------------------------------------
     @staticmethod
     def get_name():
-        return _("Project")
+        return _("Projects")
 
     def get_description(self):
         return _("Create Spyder projects and manage their files.")
@@ -385,6 +385,7 @@ class Projects(SpyderDockablePlugin):
     def on_close(self, cancelable=False):
         """Perform actions before parent main window is closed"""
         self.save_config()
+        self.watcher.stop()
         return True
 
     def unmaximize(self):
